using System.Diagnostics;
using System;
using System.Windows.Forms;
using System.Collections;
using System.Drawing;
using Microsoft.VisualBasic;
using System.Data;
using System.Collections.Generic;
using System.IO;


namespace Best_XY_MLR_cs
{
    public partial class Form1
    {
        public Form1()
        {
            InitializeComponent();
        }
        private string sDataFilename;
        private bool bEditMode;
        private bool bTextHasChanged;

        private void cmdCalc_Click(System.Object sender, System.EventArgs e)
        {
            CStatSum objLR;
            CResults objRes;
            int I;
            int nDataCOunt = 0;
            double fShiftX;
            double fShiftY;
            double fScaleX;
            double fScaleY;
            string sBuffer;

            if (sDataFilename == "")
            {
                MessageBox.Show("Please select a data file first", "Error", MessageBoxButtons.OK, MessageBoxIcon.Hand);
                return;
            }
            objLR = new CStatSum();
            objRes = new CResults();

            if (bTextHasChanged)
            {
                if (MessageBox.Show("Save changed data?", "Confirmation", MessageBoxButtons.YesNo, MessageBoxIcon.Question) == System.Windows.Forms.DialogResult.Yes)
                {
                    File.WriteAllText(sDataFilename, txtRes.Text);
                }
                bTextHasChanged = false;
            }
            bEditMode = false;

            // check the Shift X text box
            if (txtShiftX.Text.Length > 0)
            {
                fShiftX = double.Parse(txtShiftX.Text);
            }
            else
            {
                fShiftX = 0;
            }

            // check the Shift Y text box
            if (txtShiftY.Text.Length > 0)
            {
                fShiftY = double.Parse(txtShiftY.Text);
            }
            else
            {
                fShiftY = 0;
            }

            // check the Scale X text box
            if (txtScaleX.Text.Length > 0)
            {
                fScaleX = double.Parse(txtScaleX.Text);
                if (fScaleX == 0)
                {
                    fScaleX = 1;
                }
            }
            else
            {
                fScaleX = 1;
            }

            // check the Scale Y text box
            if (txtScaleY.Text.Length > 0)
            {
                fScaleY = double.Parse(txtScaleY.Text);
                if (fScaleY == 0)
                {
                    fScaleY = 1;
                }
            }
            else
            {
                fScaleY = 1;
            }

            if (objLR.GetData(sDataFilename, ref nDataCOunt, fShiftX, fShiftY, fScaleX, fScaleY))
            {
                Cursor = Cursors.WaitCursor;
                objLR.FindBestFit(ref objRes);
                objRes.SortResults();
                sBuffer = "Source Data File: " + sDataFilename + "\r\n" + "\r\n";
                sBuffer = sBuffer + "Date/Time: " + DateTime.Now + "\r\n" + "\r\n";
                sBuffer = sBuffer + "Number of observations = " + nDataCOunt + "\r\n" + "\r\n";
                if (fScaleX != 1)
                {
                    sBuffer = sBuffer + "Scale X = " + fScaleX + "\r\n";
                }
                if (fShiftX != 0)
                {
                    sBuffer = sBuffer + "Shift X = " + fShiftX + "\r\n";
                }
                if (fScaleY != 1)
                {
                    sBuffer = sBuffer + "Scale Y = " + fScaleY + "\r\n";
                }
                if (fShiftY != 0)
                {
                    sBuffer = sBuffer + "Shift Y = " + fShiftY + "\r\n" + "\r\n";
                }
                for (I = 0; I <= objRes.Count() - 1; I++)
                {
                    sBuffer = sBuffer + "R-Sqr = " + objRes.GetR2(I).ToString() + "\r\n";
                    sBuffer = sBuffer + "Model: " + objRes.GetModel(I) + "\r\n";
                    sBuffer = sBuffer + "A  = " + objRes.GetIntercept(I).ToString() + ", B1 = " + objRes.GetSlope1(I).ToString() + ", B2 = " + objRes.GetSlope2(I).ToString() + "\r\n";
                }
                txtRes.Text = sBuffer;
                sBuffer = "";
                Cursor = Cursors.Default;
                cmdSaveRes.Enabled = true;
            }
            else
            {
                MessageBox.Show("Error in processing data", "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }


        }

        private void Form1_Load(System.Object sender, System.EventArgs e)
        {
            sDataFilename = "";
            cmdCalc.Enabled = false;
            cmdSaveRes.Enabled = false;
            bEditMode = false;
        }

        private void cmdReadData_Click(System.Object sender, System.EventArgs e)
        {
            dlgReadData.Filter = "All files (*.*)|*.*|Text files|*.txt|Data files (*.dat)|*.dat";
            if (dlgReadData.ShowDialog() == System.Windows.Forms.DialogResult.OK)
            {
                sDataFilename = dlgReadData.FileName;
                txtRes.Text = File.ReadAllText(sDataFilename);
                cmdCalc.Enabled = true;
                cmdSaveRes.Enabled = true;
                bTextHasChanged = false;
                bEditMode = true;
            }
        }

        private void cmdSaveRes_Click(System.Object sender, System.EventArgs e)
        {
            dlgSaveRes.Filter = "All files (*.*)|*.*|Text files|*.txt|Data files (*.dat)|*.dat";
            if (dlgSaveRes.ShowDialog() == System.Windows.Forms.DialogResult.OK)
            {
                File.WriteAllText(dlgSaveRes.FileName, txtRes.Text);
                bTextHasChanged = false;
            }
        }

        private void cmdClose_Click(System.Object sender, System.EventArgs e)
        {
            if (MessageBox.Show("Close application?", "Confirmation", MessageBoxButtons.YesNo, MessageBoxIcon.Question) == System.Windows.Forms.DialogResult.Yes)
            {
                Close();
            }
        }

        private void txtRes_TextChanged(System.Object sender, System.EventArgs e)
        {
            if (bEditMode)
            {
                bTextHasChanged = true;
            }
        }

        private void cmdHelp_Click(System.Object sender, System.EventArgs e)
        {
            string sText;
            double x1, x2;
            int i;
            Random r = new Random();

            sText = "Each line can be 1) empty, 2) a comment line or 3) a data line" + "\r\n" + 
                    "A data line has a pair of y and x values separated by a comma" + "\r\n" + 
                    "A data line can have a weight value that is appended after x and is separated by a comma" + "\r\n" + 
                    "Weights are optional and need to appear when their values are not 1" + "\r\n" + 
                    "A comment line must NOT start with any of the chatacters +-.0123456789" + "\r\n" + "Show an example?";
            if (MessageBox.Show(sText, "Help", MessageBoxButtons.YesNo, MessageBoxIcon.Information) == System.Windows.Forms.DialogResult.Yes)
            {
                bEditMode = false;
                sText = "Sample data (example of free form comment line)" + "\r\n" + 
                        "45,32" + "\r\n" + "67,34" + "\r\n" + 
                        "Next line is an observation that is temporaryly commente dout" + "\r\n" + 
                        "! 56,23" + "\r\n";
                for (i = 1; i < 11; i++)
                {
                    x1 = (int)(200 * r.NextDouble());
                    x2 = (int)(200 * r.NextDouble());
                    sText = sText + x1.ToString() + "," + x2.ToString() + "\r\n";
                }
                txtRes.Text = sText;
            }
        }
    }

}
